unit HSExpBox;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, StdCtrls, ShellApi,
  Controls, Menus, Forms, Dialogs, IniFiles, ExtCtrls, FileCtrl, HSgrphic, HSUtils;

type
  TStringsCase = (cs_Normal, cs_Upper, cs_Lower);
  TSize = (szSmall, szBig);
  TBoxStyle = (stBox, stList);

type
  THSExplorerBox = class(TCustomListBox)
  private
    FCompyter,
    FFirst : Boolean;
    FSize : TSize;
    FBoxStyle : TBoxStyle;
    FHeader : THeader;
    FFileName : string;
    FDirectory: TFileName;
    FBmpSmall,
    FBmpBig       : array [0..50] of TBitmap;
    FDriveSmall,
    FDriveBig     : array [0..10] of TBitMap;
    FOnFileChange,
    FOnDirChange: TNotifyEvent;
    FFilter: string;
    FStringsCase: TStringsCase;
    FIcon: TIcon;
    FMetaFile : TMetaFile;
    FDirLabel : TLabel;
    FOldColumns,
    FSectionWidth,
    FCounter,
    FDriveCounter : integer;
    FListExt,
    FListExe,
    FSizeList,
    FDrives,
    FTitles,
    FAttribList : TStringList;
    FWinDir  : string;
    procedure SetDirectory(Value: TFileName);
    procedure SetStringsCase(ACase: TStringsCase);
    procedure SetSize(ASize: TSize);
    procedure ReadIni;
    procedure SetBoxStyle(ABoxStyle : TBoxStyle);
    procedure UpdateHeader;
    procedure SetHeader(AHeader : THeader);
    procedure LoadBitmapsFromRes;
    procedure MyCompyter;
  protected
    procedure DrawItem(Index: Integer; Recty: TRect; State: TOwnerDrawState); override;
    procedure Update; virtual;
    procedure Click; override;
    procedure DblClick; override;
    procedure RePaint; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CreateWnd; override;
    procedure BuildDrivesList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function RPos(const SubStr: Char; Str: string): Word;
    property Compyter : Boolean read FCompyter write FCompyter;
    procedure Refresh; virtual;
  published
    property Directory: TFileName read FDirectory write SetDirectory;
    property FileName: string read FFileName write FFileName;
    property Filter: string read FFilter write FFilter;
    property StringsCase: TStringsCase read FStringsCase write SetStringsCase;
    property IconSize : TSize read FSize write SetSize;
    property ExplorerStyle : TBoxStyle read FBoxStyle write SetBoxStyle;
    property DirLabel : TLabel read FDirlabel write FDirLabel;
    property Header : THeader read FHeader write SetHeader;
    property Align;
    property BorderStyle;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property IntegralHeight;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnClick;
    property OnDblClick;
    property OnFileChange: TNotifyEvent read FOnFileChange write FOnFileChange;
    property OnDirChange: TNotifyEvent read FOnDirChange write FOnDirChange;
  end;

implementation

{$R *.RES}

constructor THSExplorerBox.Create(AOwner: TComponent);
var
  i, j: Integer;
  s: string;
begin
  inherited Create(AOwner);
  Style := lbOwnerDrawFixed;
  Sorted := true;
  Columns := 3;
  FFilter := '*.*';
  FStringsCase := cs_Normal;
  for i := 0 to 11 do begin
  FBmpSmall[i] := TBitmap.Create;
  FBmpBig[i] := TBitmap.Create;
  end;
  for i := 0 to 10 do begin
  FDriveSmall[i] := TBitmap.Create;
  FDriveBig[i] := TBitmap.Create;
  end;
  FDrives := TStringList.Create;
  FListExt := TStringList.Create;
  FListExe := TStringList.Create;
  FSizeList := TStringList.Create;
  FAttribList := TStringList.Create;
  FTitles := TStringList.Create;
  LoadBitmapsFromRes;
  BuildDrivesList;
  FHeader := nil;
  FBoxStyle := stBox;
  FSize := szSmall;
  ReadIni;
  FIcon := TIcon.Create;
  FMetaFile := TMetaFile.Create;
  FSectionWidth := Round(Width / 4);
  FCompyter := false;
  GetDir(0, s);
  FDirectory := s;
end;

destructor THSExplorerBox.Destroy;
var
  i: Integer;
begin
  for i := 0 to FCounter do begin
  FBmpSmall[i].Free;
  FBmpBig[i].Free; end;
  for i := 0 to FDriveCounter do begin
  FDriveSmall[i].Free;
  FDriveBig[i].Free; end;
  FListExt.Free;
  FListExe.Free;
  FIcon.Free;
  FMetaFile.Free;
  FSizeList.Free;
  FAttribList.Free;
  FDrives.Free;
  FTitles.Free;
  inherited Destroy;
end;

function THSExplorerBox.RPos(const SubStr: Char; Str: string): Word;
var
  i: Word;
begin
  Result := 0;
  repeat
    if Str[Length(Str)] = SubStr then exit else
    begin
      Dec(Str[0]);
      Inc(Result)
    end
  until Length(Str) = 0;
end;

procedure THSExplorerBox.SetDirectory(Value: TFileName);
begin
  if Value = '' then
     begin
       FCompyter := true;
       MyCompyter;
       exit;
     end;
  if Value[Length(Value)] <> '\' then
    begin
      FDirectory := Value + '\';
      FDirLabel.Caption := Value + '\';
      if FCompyter then FCompyter := false;
    end
  else
    begin
      FDirectory := Value;
      FDirLabel.Caption := Value;
      if FCompyter then FCompyter := false;
   end;
  Update;
end;

procedure THSExplorerBox.Update;

  procedure AddToLists(SRec : TSearchRec);
  begin
      FSizeList.Add(IntToStr(SRec.Size));
      case SRec.Attr of
           faHidden: begin
                        FAttribList.Add('Hidden file');
                     end;
           faReadOnly: begin
                        FAttribList.Add('Read Only');
                       end;
           faSysFile: begin
                        FAttribList.Add('System file');
                      end;
           faArchive: begin
                        FAttribList.Add('Archive');
                      end;
            else FattribList.Add('');
      end;
  end;

var
  ARec: TSearchRec;
  Res, Index, i: Integer;
  SList: TStringList;
begin
  Clear;
  Application.ProcessMessages;
  FSizeList.Clear;
  FAttribList.Clear;
  SList := TStringList.Create;
  Items.Add('..');
  FSizeList.Add('');
  FAttribList.Add('');
  Res := FindFirst(FDirectory + '*.*', faDirectory or faReadOnly or faSysFile or faHidden or faArchive, ARec);
  repeat
      case FStringsCase of
        cs_Upper: ARec.Name := UpperCase(ARec.Name);
        cs_Lower: ARec.Name := LowerCase(ARec.Name);
      end;
      if (ARec.Attr and faDirectory) > 0 then
         begin
               if (ARec.Name <> '.') and (ARec.Name <> '..') then
                  begin
                      Items.Add(ARec.Name);
                      AddToLists(ARec);
                  end;
        end
     else
        begin
            if FFilter = '*.*' then
               begin
                 SList.Add(ARec.Name);
                 AddToLists(ARec);
               end
           else
               begin
                  if Pos(ExtractFileExt(ARec.Name), FFilter) <> 0 then
                    begin
                      SList.Add(ARec.Name);
                      AddToLists(ARec);
                    end;
               end;
         end;
  Res := FindNext(ARec);
  Until Res <> 0;
  SList.Sorted := Sorted;

  for Index := 0 to SList.Count - 1 do
        Items.Insert(Items.Count, SList.Strings[Index]);

  SList.Free;
end;

procedure THSExplorerBox.Click;
var
  Res: Integer;
  AFull: string;
begin
  AFull := FDirectory + Items.Strings[ItemIndex];
  Res := FileGetAttr(AFull);
  if Res = (faArchive and faDirectory)then
    begin
      if Assigned(FOnFileChange) then FOnFileChange(Self);
    end
  else FFileName := Items.Strings[ItemIndex];
  inherited Click;
end;

procedure THSExplorerBox.DblClick;
var Res, j, i: Integer;
    DirOld, AFull, AExt, APrg, ADir, AParam: string;
    PPrg, PDir, PParam: array [0..254] of Char;
begin
  if ItemIndex = -1 then Exit;
  if FCompyter = true then
    begin
      FCompyter := false;
      FDirectory := FDrives.Strings[ItemIndex];
      Update;
      exit;
    end;
  if Items.Strings[ItemIndex] = '..' then
  begin
  for i := 0 to FDrives.Count - 1 do
    begin
      if Directory = FDrives.Strings[i] then
        begin
          FCompyter := true;
          MyCompyter;
          exit;
        end;
   end;
    Dec(FDirectory[0]);
    AFull := Copy(FDirectory, 0, Length(FDirectory) - RPos('\', FDirectory));
    DirOld := Copy(FDirectory, Length(FDirectory) - RPos('\', FDirectory) + 1,
                   Length(FDirectory));
    Directory := AFull;
    if Assigned(FOnDirChange) then FOnDirChange(Self);
    ItemIndex := Items.IndexOf(FDirectory);
    exit;
  end;
  AFull := FDirectory + Items.Strings[ItemIndex];
  Res := FileGetAttr(AFull);
  if (Res AND faDirectory) > 0 then
     begin
       Directory := AFull;
       if Assigned(FOnDirChange) then FOnDirChange(Self);
       ItemIndex := 0
     end
  else
     begin
       AExt := Copy(extractfileext(lowercase(Items.Strings[ItemIndex])), 2, 3);
       if (AExt <> 'exe') and (AExt <> 'com') and (AExt <> 'scr') and (AExt <> 'cmd') then
           begin
             j := FListExt.IndexOf(AExt);
             if j <> - 1 then
                begin
                  APrg := FListExe.Strings[j];
                  ADir := ExtractFilePath(APrg);
                  AParam := FDirectory + Items.Strings[ItemIndex];
                  ShellExecute(Application.MainForm.Handle, nil, StrPCopy(PPrg, Aprg),
                  StrPCopy(PParam, AParam), StrPCopy(PDir, ADir), SW_SHOWNORMAL);
                end
            else inherited DblClick;
          end
       else
          begin
             APrg := FDirectory + Items.Strings[ItemIndex];
             AParam := '';
             ADir := FDirectory;
             ShellExecute(Application.MainForm.Handle, nil, StrPCopy(PPrg, Aprg),
             StrPCopy(PParam, AParam), StrPCopy(PDir, ADir), SW_SHOWNORMAL);
          end;
    end;
end;

procedure THSExplorerBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = 13 then DblClick;
end;

procedure THSExplorerBox.RePaint;
begin
  inherited RePaint
end;

procedure THSExplorerBox.DrawItem(Index: Integer; Recty: TRect; State: TOwnerDrawState);

procedure DrawBitmap(Canvas: TCanvas; bmpItem : integer; Rect : TRect);
begin
    with Canvas do
      begin
        if FSize = szSmall then
          begin
            BrushCopy(Bounds(Rect.Left, Rect.Top, 16, 16), FBmpSmall[bmpItem], Bounds(0, 0, 16, 16), Brush.Color);
          end
        else
          begin
            BrushCopy(Bounds(Rect.Left + ((Rect.Right - Rect.Left) div 2) - 16, Rect.Top + 1, 32, 32),
                      FBmpBig[bmpItem], Bounds(0, 0, 32, 32), Brush.Color);
          end;
      end;
end;

procedure DrawText(Canvas: TCanvas; Index: integer; Rect : TRect; sPoint : integer; Text : string);
begin
  with Canvas do
    begin
    if FSize = szSmall then
       begin
           if sPoint = 0 then TextOut(Rect.Left + + ItemHeight + 5, Rect.Top, '[' + Items.Strings[Index] + ']')
           else TextOut(Rect.Left + + ItemHeight + 5, Rect.Top, Items.Strings[Index]);
           if FBoxStyle = stList then
              begin
                TextOut(Rect.Left + FSectionWidth, Rect.Top, FSizeList.Strings[Index] + '  byte');
                TextOut(Rect.Left + 2 * FSectionWidth, Rect.Top, Text);
                TextOut(Rect.Left + 3 * FSectionWidth, Rect.Top, FAttribList.Strings[Index]);
              end;
       end
    else
       begin
          if sPoint = 0 then TextOut(Recty.Left + (((Recty.Right - Recty.Left) div 2)
               - TextWidth(Items[Index]) div 2), Recty.Top + 36, '[' + Items[Index] + ']')
          else  TextOut(Recty.Left + (((Recty.Right - Recty.Left) div 2)
               - TextWidth(Items[Index]) div 2), Recty.Top + 36, Items[Index]);
       end;
    end;
end;

var
  i, j, Res, Hndl: Integer;
  PFileName: array [0..255] of Char;
  IcnRecty: TRect;
  Ext, ProgName : string;
begin
  with Canvas do
    begin
      if FSize = szSmall then
      FillRect(Bounds(Recty.Left, Recty.Top + 1, 16, 16))
      else FillRect(Bounds(Recty.Left + ((Recty.Right - Recty.Left) div 2) - 16, Recty.Top + 1, 32, 32));
      if State = [odFocused] then
        begin
         DrawFocusRect(Recty);
        end;
      if State = [odFocused, odSelected] then
        begin
         DrawFocusRect(Recty);
         DrawFocusRect(Recty);
         DrawFocusRect(Recty);
        end;
    if FCompyter = false then
       begin
       Res := FileGetAttr(FDirectory + Items.Strings[Index]);
       if (Res and faDirectory) > 0 then
          begin
            if Items.Strings[Index] = '..' then
               begin
                 DrawBitmap(Canvas, 4, Recty);
                 exit;
               end;
            DrawBitmap(Canvas, 0, Recty);
            DrawText(Canvas, Index, Recty, 0, 'File Folder');
            exit;
          end;
       if Pos('.wmf', LowerCase(Items[Index])) <> 0 then
          begin
             try
               begin
               FMetaFile.LoadFromFile(FDirectory + Items[Index]);
               if FSize = szSmall then begin
                  FBmpSmall[3] := GetBitmapFromMetaFile(FMetaFile, 16);
                  DrawBitmap(Canvas, 3, Recty);
                  DrawText(Canvas, Index, Recty, 1, 'Metafile');
                  FBmpSmall[3].Free;
                  FBmpSmall[3] := nil;
               end else begin
                  FBmpBig[3] := GetBitmapFromMetaFile(FMetaFile, 32);
                  DrawBitmap(Canvas, 3, Recty);
                  DrawText(Canvas, Index, Recty, 1, 'Metafile');
                  FBmpBig[3].Free;
                  FBmpBig[3] := nil; end;
               end;
             except
               begin
               showmessage('Bad file data');
               exit;
               end;
             end;
             exit;
          end;
       if Pos('.ico', LowerCase(Items[Index])) <> 0 then
          begin
             try
               begin
               FIcon.LoadFromFile(FDirectory + Items[Index]);
               if FSize = szSmall then begin
                  FBmpSmall[3] := GetBitmapFromIcon(FIcon, clWhite, 16);
                  DrawBitmap(Canvas, 3, Recty);
                  DrawText(Canvas, Index, Recty, 1, 'Icon file');
                  FBmpSmall[3].Free;
                  FBmpSmall[3] := nil;
               end else begin
                  FBmpBig[3] := GetBitmapFromIcon(FIcon, clWhite, 32);
                  DrawBitmap(Canvas, 3, Recty);
                  DrawText(Canvas, Index, Recty, 1, 'Icon file');
                  FBmpBig[3].Free;
                  FBmpBig[3] := nil; end;
               end;
             except
               begin
               showmessage('Bad file data');
               exit;
               end;
             end;
             exit;
          end;
       if Pos('.scr', LowerCase(Items[Index])) <> 0 then
          begin
             DrawBitmap(Canvas, 2, Recty);
             DrawText(Canvas, Index, Recty, 1, 'Screen saver');
             exit;
          end;
       if (Pos('.com', LowerCase(Items[Index])) <> 0) or (Pos('.bat', LowerCase(Items[Index])) <> 0) then
          begin
             DrawBitmap(Canvas, 5, Recty);
             DrawText(Canvas, Index, Recty, 1, 'DOS program');
             exit;
          end;
       if (Pos('.dxf', LowerCase(Items[Index])) <> 0) then
          begin
             DrawBitmap(Canvas, 11, Recty);
             DrawText(Canvas, Index, Recty, 1, 'AutoCAD file');
             exit;
          end;
       if Pos('.drv', LowerCase(Items[Index])) <> 0 then
          begin
             DrawBitmap(Canvas, 7, Recty);
             DrawText(Canvas, Index, Recty, 1, 'Device driver file');
             exit;
          end;
       if (Pos('.sys', LowerCase(Items[Index])) <> 0) or (Pos('.dll', LowerCase(Items[Index])) <> 0)
         or (Pos('.lib', LowerCase(Items[Index])) <> 0) then
          begin
             DrawBitmap(Canvas, 8, Recty);
             DrawText(Canvas, Index, Recty, 1, 'System file');
             exit;
          end;
       if (Pos('.ttf', LowerCase(Items[Index])) <> 0) then
          begin
             DrawBitmap(Canvas, 9, Recty);
             DrawText(Canvas, Index, Recty, 1, 'True Type Font');
             exit;
          end;
       if (Pos('.dbf', LowerCase(Items[Index])) <> 0) or (Pos('.db', LowerCase(Items[Index])) <> 0) then
          begin
             DrawBitmap(Canvas, 10, Recty);
             DrawText(Canvas, Index, Recty, 1, 'Data base file');
             exit;
          end;
       if Pos('.exe', LowerCase(Items[Index])) <> 0 then
          begin
            StrPCopy(PFileName, FDirectory + Items.Strings[Index]);
            FIcon.Handle := ExtractIcon(Application.Handle, PFileName, 0);
            if (FIcon.Handle <> 0) AND (FIcon.Handle <> 1) then
               begin
                  if FSize = szSmall then begin
                  FBmpSmall[3] := GetBitmapFromIcon(FIcon, clWhite, 16);
                  DrawBitmap(Canvas, 3, Recty);
                  DrawText(Canvas, Index, Recty, 1, 'Executable file 16-bit');
                  FBmpSmall[3].Free;
                  FBmpSmall[3] := nil;
                  end else begin
                  FBmpBig[3] := GetBitmapFromIcon(FIcon, clWhite, 32);
                  DrawBitmap(Canvas, 3, Recty);
                  DrawText(Canvas, Index, Recty, 1, 'Executable file 16-bit');
                  FBmpBig[3].Free;
                  FBmpBig[3] := nil;
                  end;
                  exit;
                end
            else
                begin
                   Hndl := LoadLibrary(PFileName);
                   case Hndl of
                   20: begin
                         DrawBitmap(Canvas, 5, Recty);
                         DrawText(Canvas, Index, Recty, 1, 'DOS program');
                       end;
                   21: begin
                         DrawBitmap(Canvas, 2, Recty);
                         DrawText(Canvas, Index, Recty, 1, 'Executable file 32-bit');
                        end;
                    else
                         DrawBitmap(Canvas, 6, Recty);
                         DrawText(Canvas, Index, Recty, 1, 'Executable file 16-bit');
                    end;
                    FreeLibrary(Hndl);
                    exit;
               end;
       end;
       if (Pos('.pcd', LowerCase(Items[Index])) <> 0) or (Pos('.jpg', LowerCase(Items[Index])) <> 0)
          or (Pos('.pcx', LowerCase(Items[Index])) <> 0) or (Pos('.tif', LowerCase(Items[Index])) <> 0)
          or (Pos('.pic', LowerCase(Items[Index])) <> 0) or (Pos('.jpe', LowerCase(Items[Index])) <> 0)
          or (Pos('.gif', LowerCase(Items[Index])) <> 0) or (Pos('.tga', LowerCase(Items[Index])) <> 0)
          or (Pos('.img', LowerCase(Items[Index])) <> 0) or (Pos('.rle', LowerCase(Items[Index])) <> 0) then
          begin
             DrawBitmap(Canvas, 11, Recty);
             DrawText(Canvas, Index, Recty, 1, 'Picture file');
          end;
       ext := Copy(ExtractFileExt(lowercase(Items[Index])), 2, 3);
        j := FListExt.IndexOf(Ext);
       if j = - 1 then
         begin
             DrawBitmap(Canvas, 1, Recty);
             DrawText(Canvas, Index, Recty, 1, 'Unknown file');
             exit;
         end
       else
         begin
             ProgName := ExtractFileName(FListExe.Strings[j]);
             ProgName := Copy(ProgName, 0, Pos('.', ProgName) - 1);
             DrawBitmap(Canvas, j + 12, Recty);
             DrawText(Canvas, Index, Recty, 1, ProgName + ' file');
             exit;
         end;

    if FBmpSmall[3] = nil then FBmpSmall[3] := TBitmap.Create;
    if FBmpBig[3]= nil then FBmpBig[3] := TBitmap.Create;
   end
 else
   begin
      if FSize = szSmall then
         begin
            BrushCopy(Bounds(Recty.Left, Recty.Top, 16, 16), FDriveSmall[Index], Bounds(0, 0, 16, 16), Brush.Color);
            TextOut(Recty.Left + + ItemHeight + 5, Recty.Top, Items[Index]);
         end
      else
         begin
            BrushCopy(Bounds(Recty.Left + ((Recty.Right - Recty.Left) div 2) - 16, Recty.Top + 1, 32, 32),
                  FDriveBig[Index], Bounds(0, 0, 32, 32), Brush.Color);
            TextOut(Recty.Left + (((Recty.Right - Recty.Left) div 2)
               - TextWidth(Items[Index]) div 2), Recty.Top + 36, Items[Index]);
         end;
   end;
 end;
end;

procedure THSExplorerBox.CreateWnd;
var
  s: string;
begin
  inherited CreateWnd;
  if FCompyter = false then
    begin
      Directory := FDirectory;
      FSectionWidth := Round(Width / 4);
    end
  else
    begin
      MyCompyter;
    end;
end;

procedure THSExplorerBox.SetStringsCase(ACase: TStringsCase);
begin
  FStringsCase := ACase;
  Update;
end;

procedure THSExplorerBox.SetSize(ASize : TSize);
var
  i : integer;
begin
    if (FBoxStyle = stList) and (ASize = szBig) then
       begin
         FSize := ASize;
         ItemHeight := 39 + Canvas.TextHeight('W');
         SetBoxStyle(stBox);
         Update;
         exit;
       end;
    FSize := ASize;
    if FSize = szSmall then
      begin
        ItemHeight := 18;
      end
    else
      begin
        ItemHeight := 39 + Canvas.TextHeight('W');
      end;
end;

procedure THSExplorerBox.ReadIni;

procedure ReadIcon(Rec : string);
var
  P1: string;
  PFileName : array [0..254] of Char;
  FIcn : TIcon;
  Hndl : integer;
begin
   FCounter := FCounter + 1;
   P1 := Copy(Rec, 0, Pos('exe', lowercase(Rec)) + 2);
   if Pos(':', lowercase(P1)) = 0 then
      begin
        StrPCopy(PFileName, FWinDir + P1);
        FListExe.Add(FWinDir + P1);
      end
   else
      begin
        StrPCopy(PFileName, P1);
        FListExe.Add(P1);
      end;
   FIcn := TIcon.Create;
   Hndl := LoadLibrary(PFileName);
   try
     FIcn.Handle := ExtractIcon(Hndl, PFileName, 0);
   except
     raise EReadError.Create('Bad icon data');
     FIcn.Free;
     Exit;
   end;
   FBmpSmall[FCounter] := GetBitmapFromIcon(FIcn, clWhite, 16);
   FBmpBig[FCounter] := GetBitmapFromIcon(FIcn, clWhite, 32);
   FIcn.Free;
   FreeLibrary(Hndl);
end;

var
   i : integer;
   WinDir1 : array [0..254] of Char;
   ImeExe : string;
   WinIni : TIniFile;
begin
     GetWindowsDirectory(WinDir1, 254);
     FWinDir := StrPas(WinDir1) + '\';
     WinIni := TIniFile.Create(FWinDir + 'win.ini');
     Winini.ReadSection('Extensions', FListExt);
     FCounter := FListExt.Count;
     for i := 0 to FCounter - 1 do
       begin
          if FListExt.Strings[i] = '' then FListExt.Delete(i);
       end;
     FCounter := 11;
     for i := 0 to FListExt.Count - 1 do
       begin
          ImeExe := winini.ReadString('Extensions', FListExt.Strings[i], '');
          ReadIcon(ImeExe);
       end;
     WinIni.Free;
end;


procedure THSExplorerBox.SetBoxStyle(ABoxStyle: TBoxStyle);
begin
     FBoxStyle := ABoxStyle;
     if FBoxStyle = stBox then
       begin
         if FOldColumns > 1 then
         Columns := FOldColumns
         else Columns := 5;
        if FHeader <> nil then
          begin
            FHeader.Visible := false;
          end;
      end
    else
      begin
         FOldColumns := Columns;
         Columns := 0;
         if FSize = szBig then
           begin
             SetSize(szSmall);
             if FHeader <> nil then
               begin
                 FHeader.Visible := true;
                 Application.ProcessMessages;
                 UpdateHeader;
               end;
          end;
      end;
end;

procedure THSExplorerBox.UpdateHeader;
begin
     FHeader.Left := Left;
     FHeader.width := Width;
     FHeader.Height := Canvas.TextHeight('W') + 5;
     {if (Align = alNone) and (Top < FHeader.Top)
     then} FHeader.Top := Top - (FHeader.Height + 2);
     FHeader.Sections.Clear;
     FHeader.Sections.Add('File name');
     FHeader.Sections.Add('Size');
     FHeader.Sections.Add('Type');
     FHeader.Sections.Add('Attributs');
     FHeader.SectionWidth[0] := FSectionWidth;
     FHeader.SectionWidth[1] := FSectionWidth;
     FHeader.SectionWidth[2] := FSectionWidth;
     FHeader.SectionWidth[3] := FSectionWidth;
     FHeader.AllowResize := false;
     FHeader.Visible := true;
end;

procedure THSExplorerBox.SetHeader(AHeader : THeader);
begin
    FHeader := AHeader;
    if FHeader <> nil then
      begin
       SetBoxStyle(stList);
       FHeader.Font := Font;
       FHeader.BorderStyle := BorderStyle;
       UpdateHeader;
      end
    else SetBoxStyle(stBox);
end;

procedure THSExplorerBox.LoadBitmapsFromRes;
begin
  FBmpSmall[0].Handle := LoadBitmap(HInstance, 'EXP_FOLDER_SMALL');
  FBmpSmall[1].Handle := LoadBitmap(HInstance, 'EXP_FILE_SMALL');
  FBmpSmall[2].Handle := LoadBitmap(HInstance, 'EXP_EXE32_SMALL');
  FBmpSmall[3].Handle := LoadBitmap(HInstance, 'EXP_UPONE_SMALL');
  FBmpSmall[4].Handle := LoadBitmap(HInstance, 'EXP_UPONE_SMALL');
  FBmpSmall[5].Handle := LoadBitmap(HInstance, 'EXP_DOS_SMALL');
  FBmpSmall[6].Handle := LoadBitmap(HInstance, 'EXP_EXE16_SMALL');
  FBmpSmall[7].Handle := LoadBitmap(HInstance, 'EXP_DRV_SMALL');
  FBmpSmall[8].Handle := LoadBitmap(HInstance, 'EXP_SYS_SMALL');
  FBmpSmall[9].Handle := LoadBitmap(HInstance, 'EXP_TTF_SMALL');
  FBmpSmall[10].Handle := LoadBitmap(HInstance, 'EXP_DBF_SMALL');
  FBmpSmall[11].Handle := LoadBitmap(HInstance, 'EXP_FILE1_SMALL');
  FBmpBig[0].Handle := LoadBitmap(HInstance, 'EXP_FOLDER_BIG');
  FBmpBig[1].Handle := LoadBitmap(HInstance, 'EXP_FILE_BIG');
  FBmpBig[2].Handle := LoadBitmap(HInstance, 'EXP_EXE32_BIG');
  FBmpBig[3].Handle := LoadBitmap(HInstance, 'EXP_UPONE_BIG');
  FBmpBig[4].Handle := LoadBitmap(HInstance, 'EXP_UPONE_BIG');
  FBmpBig[5].Handle := LoadBitmap(HInstance, 'EXP_DOS_BIG');
  FBmpBig[6].Handle := LoadBitmap(HInstance, 'EXP_EXE16_BIG');
  FBmpBig[7].Handle := LoadBitmap(HInstance, 'EXP_DRV_BIG');
  FBmpBig[8].Handle := LoadBitmap(HInstance, 'EXP_SYS_BIG');
  FBmpBig[9].Handle := LoadBitmap(HInstance, 'EXP_TTF_BIG');
  FBmpBig[10].Handle := LoadBitmap(HInstance, 'EXP_DBF_BIG');
  FBmpBig[11].Handle := LoadBitmap(HInstance, 'EXP_FILE1_BIG');
end;


procedure THSExplorerBox.BuildDrivesList;
var
  Letter: Char;
  DriveType : TDriveType;
  title : string;
begin
  FDriveCounter := - 1;
  FDrives.Clear;
  FTitles.Clear;
  for Letter := 'A' to 'Z' do begin
    DriveType := GuessDriveType(Letter);
    if DriveType <> dtNoDrive then begin
      FDriveCounter := FDriveCounter + 1;
      case DriveType of
        dtFloppy  : begin
                      title := Letter + ': [Floppy drive]';
                      FDriveSmall[FDriveCounter] := TBitMap.Create;
                      FDriveBig[FDriveCounter] := TBitMap.Create;
                      FDriveSmall[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_FLOPPY_SMALL');
                      FDriveBig[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_FLOPPY_BIG');
                    end;
        dtCDROM   : begin
                      title := Letter + ': [CDROM drive]';
                      FDriveSmall[FDriveCounter] := TBitMap.Create;
                      FDriveBig[FDriveCounter] := TBitMap.Create;
                      FDriveSmall[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_CDDRIVE_SMALL');
                      FDriveBig[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_CDDRIVE_BIG');
                    end;
        dtFixed   : begin
                      title := Letter + ': [' + GetVolumeID(Letter) + ']';
                      FDriveSmall[FDriveCounter] := TBitMap.Create;
                      FDriveBig[FDriveCounter] := TBitMap.Create;
                      FDriveSmall[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_DRIVE_SMALL');
                      FDriveBig[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_DRIVE_BIG');
                    end;
        dtNetwork : begin
                      title := Letter + ': [' + GetNetworkVolume(Letter) + ']';
                      FDriveSmall[FDriveCounter] := TBitMap.Create;
                      FDriveBig[FDriveCounter] := TBitMap.Create;
                      FDriveSmall[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_NETDRIVE_SMALL');
                      FDriveBig[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_NETDRIVE_BIG');
                    end;
        dtRAM     : begin
                      title := Letter + ': [' + GetVolumeID(Letter) + ']';
                      FDriveSmall[FDriveCounter] := TBitMap.Create;
                      FDriveBig[FDriveCounter] := TBitMap.Create;
                      FDriveSmall[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_CDDRIVE_SMALL');
                      FDriveBig[FDriveCounter].Handle := LoadBitmap(hInstance, 'EXP_CDDRIVE_BIG');
                    end;
      end;
      case FStringsCase of
        cs_Upper: title := UpperCase(title);
        cs_Lower: title := LowerCase(title);
      end;
      FTitles.Add(title);
      FDrives.Add(Letter + ':\');
    end;
  end;
end;

procedure THSExplorerBox.MyCompyter;
var i : integer;
begin
    Clear;
    for i := 0 to FTitles.Count - 1 do
      begin
         Items.Add(FTitles.Strings[i]);
      end;
end;

procedure THSExplorerBox.Refresh;
begin
  Update;
end;

initialization
DetectCompyterDrives;
end.
